/*
    Copyright (c) 2008 by amdfanatyk <amdfanatyk@o2.pl>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#define CAPTION QString("QPreview")
#define VERSION QString("1.0")
#define AUTHOR QString("amdfanatyk@o2.pl")

void frmPreview::init()
{
	lblImage->setAcceptDrops(true);
	lblImage->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);

	connect(lblImage, SIGNAL(leftMouseButtonClicked()), this, SLOT(onImageClicked()));
	connect(lblImage, SIGNAL(droppedTextReceived(const QString &)), this, SLOT(onFileDropped(const QString &)));

	settings.setPath("amdfanatyk", "Software");
	settings.beginGroup(CAPTION);
	
	const QRect geo = QDesktopWidget().screenGeometry();

	settings.beginGroup("Geometry");
	setGeometry(
			settings.readNumEntry("x", ((geo.width() - 400) / 2)),
			settings.readNumEntry("y", ((geo.height() - 400) / 2)),
			settings.readNumEntry("w", 400),
			settings.readNumEntry("h", 400)
		);
	settings.endGroup();
}

void frmPreview::destroy()
{
	settings.beginGroup("Geometry");
	settings.writeEntry("x", geometry().x());
	settings.writeEntry("y", geometry().y());
	settings.writeEntry("w", geometry().width());
	settings.writeEntry("h", geometry().height());
	settings.endGroup();
}

void frmPreview::setDir(const QString & s)
{
	recentImage = image.filePath();

	image.setFile(s);
	dir = QFileInfo(s).dir();

	getImages();

	for (int i = 0 ; i != (int)images.count() ; i++)
	{
		if (image.filePath() == QFileInfo(dir.path() + "/" + images[i]).filePath())
		{
			sldImage->setValue(i);

			showImage();

			break;
		}
	}

	if (!images.count())
	{
		image.setFile("");

		setCaption(CAPTION + " " + VERSION + " by " + AUTHOR);
		lblImage->setText("Drop image.");
	}

	conn();
}

void frmPreview::getImages()
{
	dir.setFilter(QDir::Files | QDir::Readable);
	dir.setNameFilter("*.jpg *.jpeg *.png *.gif *.bmp *.pbm *.pgm *.ppm *.xbm *.xpm");
	dir.setSorting(QDir::Name);

	images = dir.entryList();

	int max = images.count();

	if (max > 0)
		max--;

	sldImage->setRange(0, max);
}

void frmPreview::showImage()
{
	if (!images.count())
		return;

	image.setFile(dir.path() + "/" + images[sldImage->value()]);

	if (image.filePath() != recentImage)
	{
		recentImage = image.filePath();
		realImage.load(image.filePath());
	}

	lblImage->setPixmap(realImage.smoothScale(lblImage->size(), QImage::ScaleMin));

	setCaption(image.fileName());
}

void frmPreview::conn(bool v)
{
	if (v)
		connect(sldImage, SIGNAL(valueChanged(int)), this, SLOT(showImage()));
	else
		sldImage->disconnect(SIGNAL(valueChanged(int)));
}

void frmPreview::onSldImagePressed()
{
	conn(false);
}

void frmPreview::onSldImageReleased()
{
	conn();

	showImage();
}

void frmPreview::onImageClicked()
{
	if (image.isFile())
	{
		QTextDrag * td = new QTextDrag(image.filePath(), this);
		td->setPixmap(realImage.smoothScale(48, 48));
		td->drag();
	}
}

void frmPreview::onFileDropped(const QString & s)
{
	conn(false);

	setDir(QString(s).remove("file://"));
}

void frmPreview::resizeEvent(QResizeEvent * e)
{
	Q_UNUSED(e);

	lblImage->resize((grpImage->width() - (2 * lblImage->x())), (grpImage->height() - (2 * lblImage->y())));

	showImage();
}
